@@ -13,7 +13,7 @@ epcreg <- function(formula, data
   , baselearner.control=epcreg.baselearner.control()
   , integrator.control=epcreg.integrator.control()
   , ncores=1, filemethod=FALSE, print.level=1
-  , preschedule = FALSE
+  , preschedule = TRUE
   , schedule.method = c("random", "as.is", "task.length"), task.length) {
   if (integrator.control$method!="default") stop("invalid PCR integration method")
   ncores.max <- try(detectCores(),silent=T)
@@ -71,64 +71,72 @@ plot.epcreg <- function(x, ...) {
   par(oldpar)
 }
 
-epcreg.save <- function(obj, file) {
-  if (!identical(class(obj),c("epcreg","epcreg.file"))) stop("invalid object class (must be epcreg & epcreg.file)")
-  if (missing(file)) stop("must provide file argument")
+epcreg.save <- function (obj, file) {
+  if (!("epcreg" %in% class(obj))) 
+    stop("invalid object class (must be epcreg & epcreg.file)")
+  if (missing(file)) 
+    stop("must provide file argument")
   tmpfiles <- obj$est$baselearner.cv.batch@tmpfiles
-  tmpfile.new <- tempfile()
-  save(obj, file=tmpfile.new, compress=F)
-  all.files <- c(tmpfile.new, tmpfiles)
-  all.files.basename <- basename(all.files)
   
-  tmpdir <- paste0("./.", basename(tempfile("dir")),"/")
-  dir.create(tmpdir)
-  all.files.new <- paste0(tmpdir, all.files.basename)
-  file.copy(all.files, all.files.new)
-  meta <- list(filename.mainobj=all.files.basename[1], filenames.batchobj=all.files.basename[1+1:length(tmpfiles)])
-  save(meta, file=paste0(tmpdir, "meta"), compress=FALSE)
-
-  tar(file, files=tmpdir, compression="gzip")
-  unlink(tmpdir, recursive=TRUE)
+  if (is.null(tmpfiles)) { # ordinary save
+    save(obj, file = file)
+  } else {
+    tmpfile.new <- tempfile()
+    save(obj, file = tmpfile.new, compress = F)
+    all.files <- c(tmpfile.new, tmpfiles)
+    all.files.basename <- basename(all.files)
+    tmpdir <- paste0("./.", basename(tempfile("dir")), "/")
+    dir.create(tmpdir)
+    all.files.new <- paste0(tmpdir, all.files.basename)
+    file.copy(all.files, all.files.new)
+    meta <- list(filename.mainobj = all.files.basename[1], filenames.batchobj = all.files.basename[1 + 1:length(tmpfiles)])
+    save(meta, file = paste0(tmpdir, "meta"), compress = FALSE)
+    tar(file, files = tmpdir, compression = "gzip")
+    unlink(tmpdir, recursive = TRUE)
+  }
 }
 
-epcreg.load <- function(file) {
-  filepaths <- untar(file, list=T)
-  basenames <- basename(filepaths)
-  dirnames <- dirname(filepaths)
-  if (length(unique(dirnames))>1) stop("unexpected multiple directories in tar filepaths")
-  metafile.index <- which(basenames=="meta")
+epcreg.load <- function (file) {
+  env <- new.env()
+  #load(file, envir = env)
+  loadret <- suppressWarnings(try(load(file, envir = env), silent = TRUE))
   
-  extdir <- dirnames[1] # this is where untar will extract the files to
-  untar(file)
-  meta <- NULL # to overcome codetools error: "no visible binding for meta"
-  load(filepaths[metafile.index]) # this will load "meta"
-  mainfile.index <- which(basenames==meta$filename.mainobj)
-  load(filepaths[mainfile.index]) # this will load "obj"
-  if (!identical(class(obj),c("epcreg","epcreg.file"))) stop("invalid object class (must be epcreg & epcreg.file)")
-  
-  basenames.ordered <- basename(obj$est$baselearner.cv.batch@tmpfiles)
-  #if (!identical(sort(basenames.ordered),sort(basenames[-c(metafile.index,mainfile.index)]))) stop("basenames mismatch")
-  filepaths.ordered <- paste(extdir, basenames.ordered, sep="/")
-  
-  # copy batch files to new tempfiles in tempdir
-  tmpfiles.new <- tempfile(rep("file", length(filepaths.ordered)))
-  # replaced file.rename with file.copy and unlink to handle cross-device cases (where . and R tmp directories are on different devices)
-  file.copy(from=filepaths.ordered, to=tmpfiles.new)
-  unlink(filepaths.ordered, recursive=TRUE)
-  
-  unlink(extdir, recursive=TRUE)
-  
-  obj$est$baselearner.cv.batch@tmpfiles <- tmpfiles.new
-  n.instance <- length(obj$est$baselearner.cv.batch@instance.list@instances)
-  for (i in 1:n.instance) {
-    partid <- obj$est$baselearner.cv.batch@instance.list@instances[[1]]@partid
-    nfold <- length(unique(obj$est$baselearner.cv.batch@instance.list@partitions[,partid]))
-    for (j in 1:nfold) {
-      obj$est$baselearner.cv.batch@fitobj.list[[i]]@fitobj.list[[j]]@est <- tmpfiles.new[obj$est$baselearner.cv.batch@tmpfiles.index.list$start[i]+j-1]
+  if (class(loadret) == "try-error") { # filemethod load
+    filepaths <- untar(file, list = T)
+    basenames <- basename(filepaths)
+    dirnames <- dirname(filepaths)
+    if (length(unique(dirnames)) > 1) 
+      stop("unexpected multiple directories in tar filepaths")
+    metafile.index <- which(basenames == "meta")
+    extdir <- dirnames[1]
+    untar(file)
+    meta <- NULL
+    load(filepaths[metafile.index])
+    mainfile.index <- which(basenames == meta$filename.mainobj)
+    load(filepaths[mainfile.index])
+    if (!identical(class(obj), c("epcreg", "epcreg.file"))) 
+      stop("invalid object class (must be epcreg & epcreg.file)")
+    basenames.ordered <- basename(obj$est$baselearner.cv.batch@tmpfiles)
+    filepaths.ordered <- paste(extdir, basenames.ordered, sep = "/")
+    tmpfiles.new <- tempfile(rep("file", length(filepaths.ordered)))
+    file.copy(from = filepaths.ordered, to = tmpfiles.new)
+    unlink(filepaths.ordered, recursive = TRUE)
+    unlink(extdir, recursive = TRUE)
+    obj$est$baselearner.cv.batch@tmpfiles <- tmpfiles.new
+    n.instance <- length(obj$est$baselearner.cv.batch@instance.list@instances)
+    for (i in 1:n.instance) {
+      partid <- obj$est$baselearner.cv.batch@instance.list@instances[[1]]@partid
+      nfold <- length(unique(obj$est$baselearner.cv.batch@instance.list@partitions[, partid]))
+      for (j in 1:nfold) {
+        obj$est$baselearner.cv.batch@fitobj.list[[i]]@fitobj.list[[j]]@est <- tmpfiles.new[obj$est$baselearner.cv.batch@tmpfiles.index.list$start[i] + j - 1]
+      }
     }
+    return(obj)
+  } else { # ordinary load
+    loadedObjects <- objects(env, all.names = TRUE)
+    stopifnot(length(loadedObjects) == 1)
+    return (env[[loadedObjects]])
   }
-
-  return (obj)
 }
 
 print.epcreg <- function(x, ...) {
